use maud::{html, DOCTYPE, Markup, Render};
use std::fmt::{Debug, UpperHex};
use std::sync::OnceLock;

pub struct DebugRender<T>(pub T);

impl<T: Debug> Render for DebugRender<T> {
    fn render_to(&self, w: &mut String) {
        format_args!("{0:#?}", self.0).render_to(w);
    }
}

pub struct Hex<T>(pub T);

impl<T: UpperHex> Render for Hex<T> {
    fn render_to(&self, w: &mut String) {
        format_args!("{0:X}", self.0).render_to(w);
    }
}

pub fn size_name(x: u64) -> String {
	if x == 0 {
		format!("0 B")
	} else if x % (1<<10) != 0 {
		format!("{} B", x)
	} else if x % (1<<20) != 0 {
		format!("{} KiB", x / (1<<10))
	} else if x % (1<<30) != 0 {
		format!("{} MiB", x / (1<<20))
	} else if x % (1<<40) != 0 {
		format!("{} GiB", x / (1<<30))
	} else if x % (1<<50) != 0 {
		format!("{} TiB", x / (1<<40))
	} else if x % (1<<60) != 0 {
		format!("{} PiB", x / (1<<50))
	} else {
		format!("{} EiB", x / (1<<60))
	}
}

pub fn render_page(title: &str, content: Markup) -> Markup {
	html! {
		(DOCTYPE)
		head {
			link rel="stylesheet" href={(http_path()) "/style.css"};
			title {
				(title)
			}
		}
		body {
			(content)
		}
	}
}

static HTTP_PATH: OnceLock<String> = OnceLock::new();

pub fn http_path() -> &'static str {
	HTTP_PATH.get().expect("HTTP_PATH should have been initialized before usage.")
}

pub fn http_path_set(path: String) {
	HTTP_PATH.set(path).expect("HTTP_PATH can only be set once.");
}
